package dev.kske.chess.board;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import dev.kske.chess.board.Piece.Color;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>PawnPromotion.java</strong><br>
 * Created: <strong>2 Nov 2019</strong><br>
 * 
 * @since Chess v0.5-alpha
 * @author Kai S. K. Engelbart
 */
public class PawnPromotion extends Move {

	private Constructor<? extends Piece> promotionPieceConstructor;

	public PawnPromotion(Position pos, Position dest, Class<? extends Piece> promotionPiece) throws NoSuchMethodException, SecurityException {
		super(pos, dest);
		promotionPieceConstructor = promotionPiece.getDeclaredConstructor(Color.class, Board.class);
		promotionPieceConstructor.setAccessible(true);
	}

	public PawnPromotion(int xPos, int yPos, int xDest, int yDest, Class<? extends Piece> promotionPiece)
			throws NoSuchMethodException, SecurityException {
		this(new Position(xPos, yPos), new Position(xDest, yDest), promotionPiece);
	}

	@Override
	public void execute(Board board) {
		try {
			board.set(pos, promotionPieceConstructor.newInstance(board.get(pos).getColor(), board));
			super.execute(board);
		} catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException | SecurityException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void revert(Board board, Piece capturedPiece) {
		super.revert(board, capturedPiece);
		board.set(pos, new Pawn(board.get(dest).getColor(), board));
	}
}
